#ifndef MAINNODE_H
#define MAINNODE_H

#include <nodelet/nodelet.h>
#include <ros/ros.h>
#include "boost/thread.hpp"
#include "car/environmentDataMsg.h"
#include "NotifiableThread.h"
#include "NetworkingLib/Networking.h"
#include "PlatoonController.h"

namespace car
{
class MainNode : public nodelet::Nodelet
{
public:
    virtual void onInit();

    MainNode(ros::NodeHandle & nh, std::string & name);

    MainNode();

    ~MainNode();

private:
    std::function<void()> platoonControllerNotify =
        [this]
        { platoonControllerThread.notify(); };

    NotifiableThread::Callback platoonControllerRun =
        [this]
        { platoonController.run(); };

    ros::NodeHandle nh;
    std::string name;
    boost::thread loggerModule;
    ros::Publisher logEnabled;
    ros::Subscriber environmentData;

    networking::Networking net;

    //Networking platoonProtocolThread;
    NotifiableThread platoonControllerThread;

    PlatoonController platoonController;

    void environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg);
};
}
#endif
